#include <Keyboard.h>

#include <HID.h>

#define X_AXIS_STEP_PIN 2
#define Y_AXIS_STEP_PIN 3
#define Z_AXIS_STEP_PIN 4

#define X_AXIS_DIR_PIN 5
#define Y_AXIS_DIR_PIN 6
#define Z_AXIS_DIR_PIN 7

#define STEPPER_ENABLE_PIN 8  // active-low (i.e. LOW turns on the drivers)
#define X_AXIS_LIMIT 9
#define Y_AXIS_LIMIT 10

// ================================================================================
// Import the third-party AccelStepper library.
#include <AccelStepper.h>

#include <Servo.h>

Servo myservo;  // create servo object to control

// Declare three AccelStepper objects to manage the output timing.
AccelStepper xaxis(AccelStepper::DRIVER, X_AXIS_STEP_PIN, X_AXIS_DIR_PIN);
AccelStepper yaxis(AccelStepper::DRIVER, Y_AXIS_STEP_PIN, Y_AXIS_DIR_PIN);
AccelStepper zaxis(AccelStepper::DRIVER, Z_AXIS_STEP_PIN, Z_AXIS_DIR_PIN);

long xpos = 0;
long ypos = 0;
long zpos = 0;

void setMaxSpd(float speed) {
  xaxis.setMaxSpeed(speed);
  yaxis.setMaxSpeed(speed);
  zaxis.setMaxSpeed(speed);
}

void setAccel(float accel) {
  xaxis.setAcceleration(accel);
  yaxis.setAcceleration(accel);
  zaxis.setAcceleration(accel);
}

void resetPos() {
  xpos = 0;
  ypos = 0;
  zpos = 0;
}

void moveToPos(long x, long y, long z) {
  move(x - xpos, y - ypos, z - zpos);
  xpos = x;
  ypos = y;
  zpos = z;
  Serial.print("X - "); 
  Serial.print(xpos);
  Serial.print("\tY - ");
  Serial.print(ypos);
  Serial.print("\tZ - ");
  Serial.print(zpos);
  Serial.print("\n");
}

// ================================================================================
/// Configure the hardware once after booting up.  This runs once after powering
/// up the board, pressing reset, or connecting to the console serial port.
void setup(void)
{
  // set up the CNC Shield I/O
  digitalWrite(STEPPER_ENABLE_PIN, HIGH); // initialize drivers in disabled state
  pinMode(STEPPER_ENABLE_PIN, OUTPUT);
  pinMode(X_AXIS_LIMIT, INPUT_PULLUP);
  pinMode(Y_AXIS_LIMIT, INPUT_PULLUP);
  pinMode(11, OUTPUT);
  pinMode(A1, INPUT);

  randomSeed(analogRead(A1));

  setMaxSpd(2500.0);
  setAccel(800.0);

  // set up the serial port for debugging output
  Serial.begin(115200);

  // enable the drivers, the motors will remain constantly energized
  digitalWrite(STEPPER_ENABLE_PIN, LOW);

  resetPos();
}
/****************************************************************/
/// Call the ``run()`` function for each stepper driver object, which will
/// recalculate speeds and generate step pulses as needed.
void poll_steppers(void)

{
  xaxis.run();
  yaxis.run();
  zaxis.run();
}
/// Return true if any one of the drivers are still moving.
bool is_moving(void)
{
  return (xaxis.isRunning() || yaxis.isRunning() || zaxis.isRunning());
}

/// Move a relative displacement at the current speed, blocking until the move is done.
void move(long x, long y, long z)
{
  xaxis.move(x);
  yaxis.move(y);
  zaxis.move(z);
  do {
    poll_steppers();
  } while(is_moving());
}

/****************************************************************/
/// Run one iteration of the main loop.  The Arduino system will call this
/// function over and over forever.  This implementation is a script which will
/// generate a series of movements, waiting for each to complete.
void loop()
{
  drop();
  delay(500);
  moveToPos(0, 0, 0);
  delay(random(5000) + 1000);
  moveToPos(random(27000) + 3000, random(20000) + 10000, 0);
  grab();
  moveToPos(0, 0, 0);
  delay(100);
}

void drop() {
  digitalWrite(11, HIGH);
}

void grab() 
{
  setMaxSpd(900.0);
  moveToPos(xpos, ypos, -4750);
  delay(1000);
  digitalWrite(11, LOW);
  delay(800);
  moveToPos(xpos, ypos, 0);
  setMaxSpd(2500.0);
}
/****************************************************************/
